// Copyright (C) 2012-2015 ChaosForge Ltd
// http://chaosforge.org/
//
// This file is part of Nova libraries. 
// For conditions of distribution and use, see copying.txt file in root folder.

#ifndef NV_LUA_GLM_HH
#define NV_LUA_GLM_HH

#include <nv/common.hh>
#include <nv/stl/math.hh>
#include <nv/lua/lua_values.hh>

namespace nv
{
	namespace lua
	{
		void register_glm( lua_State* L );

		namespace detail
		{
			template< typename T > inline const char* glm_metatable_name() { static_assert(sizeof(T) == 0, "Type not implemented!"); return NULL; }
			template<> inline const char* glm_metatable_name< nv::ivec2 >() { return "ivec2"; }
			template<> inline const char* glm_metatable_name< nv::ivec3 >() { return "ivec3"; }
			template<> inline const char* glm_metatable_name< nv::ivec4 >() { return "ivec4"; }
			template<> inline const char* glm_metatable_name< nv::vec2  >() { return "vec2"; }
			template<> inline const char* glm_metatable_name< nv::vec3  >() { return "vec3"; }
			template<> inline const char* glm_metatable_name< nv::vec4  >() { return "vec4"; }

			template< typename T >
			bool is_vec( lua_State* L, int index )
			{
				return is_userdata( L, index, detail::glm_metatable_name<T>() );
			}

			template< typename T >
			T to_vec( lua_State* L, int index )
			{
				return *reinterpret_cast<T*>( check_userdata( L, index, detail::glm_metatable_name<T>() ) );
			}

			template< typename T >
			T to_vec( lua_State* L, int index, const T& def )
			{
				return ( is_vec<T>( L, index ) ? *reinterpret_cast<T*>( to_pointer( L, index ) ) : def );
			}

			template< typename T >
			T* to_pvec( lua_State* L, int index )
			{
				return reinterpret_cast<T*>( check_userdata( L, index, detail::glm_metatable_name<T>() ) );
			}

			template< typename T >
			void push_vec( lua_State* L, const T& v )
			{
				push_userdata( L, v, detail::glm_metatable_name<T>() );
			}
		}

		template< typename T >
		struct pass_traits< tvec2<T> >
		{
			typedef tvec2<T> value_type;
			static void push( lua_State *L, const value_type& p ) { detail::push_vec< value_type >( L, p ); }
			static value_type to( lua_State *L, int index ) { return detail::to_vec< value_type >( L, index ); }
			static value_type to( lua_State *L, int index, const value_type& def ) { return detail::to_vec< value_type >( L, index, def ); }
		};

		template< typename T >
		struct pass_traits< tvec3<T> >
		{
			typedef tvec3<T> value_type;
			static void push( lua_State *L, const value_type& p ) { detail::push_vec< value_type >( L, p ); }
			static value_type to( lua_State *L, int index ) { return detail::to_vec< value_type >( L, index ); }
			static value_type to( lua_State *L, int index, const value_type& def ) { return detail::to_vec< value_type >( L, index, def ); }
		};

		template< typename T >
		struct pass_traits< tvec4<T> >
		{
			typedef tvec4<T> value_type;
			static void push( lua_State *L, const value_type& p ) { detail::push_vec< value_type >( L, p ); }
			static value_type to( lua_State *L, int index ) { return detail::to_vec< value_type >( L, index ); }
			static value_type to( lua_State *L, int index, const value_type& def ) { return detail::to_vec< value_type >( L, index, def ); }
		};
	}

}


#endif // NV_LUA_GLM_HH
